package com.perforce.ts.jiraAccess;

import com.atlassian.jira.rest.client.api.JiraRestClient;
import com.atlassian.jira.rest.client.api.RestClientException;
import com.atlassian.jira.rest.client.api.domain.Issue;
import com.atlassian.jira.rest.client.api.domain.SearchResult;
import com.atlassian.jira.rest.client.api.domain.util.ErrorCollection;
import com.atlassian.jira.rest.client.internal.async.AsynchronousJiraRestClientFactory;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Retrieves issue(s) from jira
 *
 * @author jbrown
 */
public class JiraRestAccess {

    public JiraRestAccess() {
    }
    private final AsynchronousJiraRestClientFactory factory = new AsynchronousJiraRestClientFactory();

    // The JIRA REST client.
    private JiraRestClient restClient;

    // Max issues to retrieve at a time. 
    private int batchSize = 100;

    /**
     * Setup the Rest Connection. This includes the user/password login.
     *
     * @param jiraUrl
     * @param jiraUser
     * @param jiraPassword
     * @throws JiraAccessException
     */
    public final void init(String jiraUrl, String jiraUser, String jiraPassword) throws JiraAccessException {
        try {
            URI jiraServerUri = new URI(jiraUrl);
            this.restClient = factory.createWithBasicHttpAuthentication(jiraServerUri, jiraUser, jiraPassword);
        } catch (Exception e) {
            // DIE!!
            throw new JiraAccessException("init setup fail: " + e.getMessage(), e);
        }
    }

    public final void setBatchSize(int batchSize) {
        this.batchSize = batchSize;
    }

    public final int getBatchSize() {
        return this.batchSize;
    }

    public DiffBugsRestDefectResult getIssue(String issueName) throws JiraAccessException {
        if (restClient == null) {
            throw new RuntimeException("getIssue called but the Rest Client was not setup by first calling init().");
        }
        DiffBugsRestDefectResult ri = null;

        try {

            Issue issue = getIssueInternal(issueName);
            if (issue != null) {
                ri = convertToDiffBugsRestDefectResult(issue);
            }

        } catch (RestClientException ex) {
            throw new JiraAccessException(ex.getMessage(), ex);
        }
        return ri;
    }

    /**
     * Retrieves the issues given. Note that jira throws a stinkin' exception if just one of the
     * issues does not exist.
     *
     * @param ids
     * @return the LIst of Issues found
     * @throws JiraAccessException
     */
    public List<DiffBugsRestDefectResult> getIssues(String[] ids) throws JiraAccessException {
        if (restClient == null) {
            throw new RuntimeException("getIssues called but the Rest Client was not setup by first calling init().");
        }
        List<DiffBugsRestDefectResult> ri = new ArrayList();
        try {

            if (ids.length < 1) {
                return ri;
            } else if (ids.length < 2) {
                ri.add(getIssue(ids[0]));
            } else {
                // more than 1 issue:  build the jql query
                StringBuilder jql = new StringBuilder("issue in (");
                for (int i = 0; i < ids.length; i++) {
                    if (i > 0) {
                        jql.append(",");
                    }
                    jql.append(ids[i]);
                }
                jql.append(")");

                /**
                 * put returned issues into a Map to weed out duplications.
                 * Duplicates can happen if the pagesize (batchCnt) overlaps on the last page:  see the rest api's javadoc.
                 */
                Map<String, Integer> issues = new HashMap();

                int cnt = batchSize;
                int cntSoFar = 0;
                while (cnt == batchSize) {
                    SearchResult searchResult = restClient.getSearchClient().searchJql(jql.toString(), batchSize, cntSoFar, null).claim();
                    cnt = 0;
                    for (Issue issue : searchResult.getIssues()) {
                        ++cnt;
                        ++cntSoFar;
                        if (null == issues.get(issue.getKey())) {
                            ri.add(convertToDiffBugsRestDefectResult(issue));
                            issues.put(issue.getKey(), null);
                        } else {
                            System.out.println("Found dup.");
                        }
                    }
                }

            }

        } catch (RestClientException ex) {
            throw new JiraAccessException(ex.getMessage(), ex);
        }
        return ri;
    }

    /**
     * Retrieves the issue. Returns null if the call worked but the issue was not found.
     *
     * @param issueName
     * @return the Issue or null if issue is not found.
     * @throws RestClientException
     */
    private Issue getIssueInternal(String issueName) throws RestClientException {
        Issue issue = null;
        try {
            issue = restClient.getIssueClient().getIssue(issueName).claim();
        } catch (RestClientException e) {

            Collection<ErrorCollection> errs = e.getErrorCollections();
            for (ErrorCollection err : errs) {
                if (err.getStatus() == 404) {
                    return (Issue)null;
                }
            }
            throw e;
        }
        return issue;
    }

    private DiffBugsRestDefectResult convertToDiffBugsRestDefectResult(Issue issue) {
        String issueId = issue.getKey();
        DiffBugsRestDefectResult ri = new DiffBugsRestDefectResult(issueId, null, issue);
        return ri;
    }
}
